// Copyright (c) 2014-2021 NetEase, Inc.
// All right reserved.

import 'package:pigeon/pigeon.dart';

class SimpleResponse {
  final int code;

  final String? msg;

  final int? data1;

  final int? data2;

  final String? data3;

  SimpleResponse(this.code, this.msg, this.data1, this.data2, this.data3);
}

// **************************************************************************

class InitializeRequest {
  InitializeRequest(this.appKey, this.extras);

  String appKey;

  Map<String?, String?>? extras;
}

class SDKVersions {
  String? imVersion;
  String? rtcVersion;
  String? whiteboardVersion;
  String? roomKitVersion;
}

@HostApi(dartHostTestHandler: "RoomKitApi")
abstract class RoomKitApi {
  @async
  SimpleResponse initialize(InitializeRequest options);

  SDKVersions sdkVersions();
}

// **************************************************************************

@HostApi(dartHostTestHandler: "AuthServiceApi")
abstract class AuthServiceApi {

  @async
  SimpleResponse login(String account, String token);

  @async
  SimpleResponse anonymousLogin(String authorization);

  @async
  SimpleResponse logout();

  bool isLoggedIn();
}

class AuthStatusEvent {
  final String status;
  final bool isLoggedIn;

  AuthStatusEvent(this.status, this.isLoggedIn);
}

@FlutterApi()
abstract class AuthStatusEventSink {

  void onAuthStatusChanged(AuthStatusEvent event);

}

// **************************************************************************
class CreateRoomParams {
  final String roomUuid;
  final String roomName;
  final String? templateId;
  final String? password;
  final Map<String?, String?>? initialProperties;

  CreateRoomParams({
    required this.roomUuid,
    required this.roomName,
    this.templateId,
    this.password,
    this.initialProperties,
  });
}

class CreateRoomOptions {
  final bool enableRecord;
  final bool enableLive;
  final bool enableRtc;
  final bool enableChatroom;
  final bool enableWhiteboard;

  CreateRoomOptions({
    this.enableRecord = false,
    this.enableLive = false,
    this.enableRtc = true,
    this.enableChatroom = true,
    this.enableWhiteboard = false,
  });
}

class JoinRoomParams {
  final String roomUuid;
  final String userName;
  final String role;
  final String? password;
  final Map<String?, String?>? initialProperties;

  JoinRoomParams({
    required this.roomUuid,
    required this.userName,
    required this.role,
    this.password,
    this.initialProperties,
  });
}

class JoinRoomOptions {
  final int unUsed;

  JoinRoomOptions(this.unUsed);
}

class PreviewRoomParams{
  // 占位，解决编译问题
  int? value;

  PreviewRoomParams();
}

class PreviewRoomOptions{
  // 占位，解决编译问题
  int? value;

  PreviewRoomOptions();
}

class RoomRole {
  final String name;
  final int limit;
  final bool superRole;
  final RoleParams? params;

  RoomRole({
    required this.name,
    required this.limit,
    this.superRole = false,
    required this.params,
  });
}

class RoleParams {
  final RoleVideoParams? video;
  final RoleAudioParams? audio;
  final RoleScreenParams? screen;

  RoleParams({
    required this.video, required this.audio, required this.screen,
  });
}

class RoleVideoParams {
  final int width;
  final int height;
  final int fps;

  RoleVideoParams({
    required this.width,
    required this.height,
    required this.fps,
  });
}

class RoleScreenParams {
  final String videoProfile;

  RoleScreenParams(this.videoProfile);
}

class RoleAudioParams {
  final String profile;
  final String scenario;

  RoleAudioParams(this.profile, this.scenario);
}

class RoomMember {
  final String uuid;
  final String name;
  final RoomRole? role;
  final bool isAudioOn;
  final bool isVideoOn;
  final bool isInRtcChannel;
  final bool isInChatroom;
  final bool isSharingScreen;
  final bool isSharingWhiteboard;
  final Map<String?, String?>? properties;

  RoomMember({
    required this.uuid,
    required this.name,
    required this.role,
    required this.isAudioOn,
    required this.isVideoOn,
    required this.isInRtcChannel,
    required this.isInChatroom,
    required this.isSharingScreen,
    required this.isSharingWhiteboard,
    required this.properties,
  });
}

class RoomSnapshot {
  String roomUuid;
  String roomName;
  String? password;
  bool isRoomLocked;
  int rtcStartTime;
  RoomMember? localMember;
  List<RoomMember?> remoteMembers;
  Map<String?, String?>? properties;
  bool isRtcSupported;
  bool isChatroomSupported;
  bool isWhiteboardSupported;
  bool isCloudRecordSupported;
  bool isLiveSupported;

  RoomSnapshot({
    required this.roomUuid,
    required this.roomName,
    required this.password,
    required this.rtcStartTime,
    required this.localMember,
    required this.remoteMembers,
    required this.properties,
    required this.isRtcSupported,
    required this.isChatroomSupported,
    required this.isWhiteboardSupported,
    required this.isCloudRecordSupported,
    required this.isLiveSupported,
    required this.isRoomLocked,
  });
}

class JoinRoomResponse {
  final int code;

  final String? msg;

  final RoomSnapshot? snapshot;

  JoinRoomResponse(this.code, this.msg, this.snapshot);
}

@HostApi(dartHostTestHandler: "RoomServiceApiOfDart")
abstract class RoomServiceApi {

  @async
  SimpleResponse createRoom(CreateRoomParams params, CreateRoomOptions options);

  @async
  JoinRoomResponse joinRoom(JoinRoomParams params, JoinRoomOptions options);

  @async
  SimpleResponse previewRoom(PreviewRoomParams params, PreviewRoomOptions options);
}

@HostApi(dartHostTestHandler: "RoomContextApiOfDart")
abstract class RoomContextApi {

  @async
  SimpleResponse changeMyName(String roomUuid, String name);

  @async
  SimpleResponse changeMemberRole(String roomUuid, String userUuid, String role);

  @async
  SimpleResponse handOverMyRole(String roomUuid, String userUuid);

  @async
  SimpleResponse kickMemberOut(String roomUuid, String userUuid);

  @async
  SimpleResponse leaveRoom(String roomUuid);

  @async
  SimpleResponse endRoom(String roomUuid);

  @async
  SimpleResponse updateRoomProperty(String roomUuid, String key, String value, String associatedUuid);

  @async
  SimpleResponse deleteRoomProperty(String roomUuid, String key);

  @async
  SimpleResponse updateMemberProperty(String roomUuid, String userUuid, String key, String value);

  @async
  SimpleResponse deleteMemberProperty(String roomUuid, String userUuid, String key);

  @async
  SimpleResponse lockRoom(String roomUuid);

  @async
  SimpleResponse unlockRoom(String roomUuid);
}

class RoomMessage {

  final String type;

  final String? fromAccount;

  final String fromNick;

  final String content;

  final List<String?>? toAccounts;

  final int time;

  RoomMessage(this.type, this.fromAccount, this.fromNick, this.content, this.toAccounts, this.time);
}

class RoomMessageEvent {
  final String roomUuid;
  final List<RoomMessage?> messages;

  RoomMessageEvent(this.roomUuid, this.messages);
}

class PropertyEvent {
  final String roomUuid;
  final String? userUuid;
  final Map<String?, String?> properties;
  PropertyEvent(this.roomUuid, this.userUuid, this.properties);
}

class MemberRoleChangeEvent {
  final String roomUuid;
  final String userUuid;
  final RoomRole? role;
  MemberRoleChangeEvent(this.roomUuid, this.userUuid, this.role);
}

class MembersEvent {
  final String roomUuid;
  final List<RoomMember?>? members;
  final List<String?>? uuids;
  MembersEvent(this.roomUuid, this.members, this.uuids);
}

class CommonEvent {
  final String roomUuid;
  final String? userUuid;
  final int? arg1;
  final int? arg2;
  final String? sArg1;
  final String? sArg2;
  CommonEvent(this.roomUuid, this.userUuid, this.arg1, this.arg2, this.sArg1, this.sArg2);
}

class MemberToggleEvent {
  final String roomUuid;
  final String userUuid;
  final String? operateUuid;
  final bool isOn;
  MemberToggleEvent(this.roomUuid, this.userUuid,this.operateUuid, this.isOn);
}

class RoomToggleEvent{
  final String roomUuid;
  final bool isOn;
  RoomToggleEvent(this.roomUuid,this.isOn);
}

class MemberVolumeInfo {
  final String userUuid;
  final int volume;
  MemberVolumeInfo(this.userUuid, this.volume);
}

class RtcVolumeEvent {
  final String roomUuid;
  final List<MemberVolumeInfo?>? volumes;
  final int totalVolume;

  RtcVolumeEvent(this.roomUuid, this.volumes, this.totalVolume,);
}

class CreateAudioMixingOption {
  final String path;
  final int loopCount;
  final bool sendEnabled;
  final int sendVolume;
  final bool playbackEnabled;
  final int playbackVolume;
  CreateAudioMixingOption({required this.path, required this.loopCount, required this.sendEnabled,
    required this.sendVolume, required this.playbackEnabled, required this.playbackVolume});
}

class CreateAudioEffectOption{
  final String path;
  final int loopCount;
  final bool sendEnabled;
  final int sendVolume;
  final bool playbackEnabled;
  final int playbackVolume;
  CreateAudioEffectOption({required this.path, required this.loopCount, required this.sendEnabled,
    required this.sendVolume, required this.playbackEnabled, required this.playbackVolume});
}

@FlutterApi()
abstract class RoomEventSink {
  void onRoomEnd(String roomUuid, String reason);

  void onRoomPropertiesChanged(PropertyEvent event);

  void onRoomPropertiesDeleted(PropertyEvent event);

  void onMemberPropertiesChanged(PropertyEvent event);

  void onMemberPropertiesDeleted(PropertyEvent event);

  void onMemberRoleChanged(MemberRoleChangeEvent event);

  void onMemberNameChanged(CommonEvent event);

  void onMemberJoinRoom(MembersEvent event);

  void onMemberLeaveRoom(MembersEvent event);

  void onMemberJoinRtcChannel(MembersEvent event);

  void onMemberLeaveRtcChannel(MembersEvent event);

  void onRtcChannelError(CommonEvent event);

  void onMemberJoinChatroom(MembersEvent event);

  void onMemberLeaveChatroom(MembersEvent event);

  void onMemberAudioMuteChanged(MemberToggleEvent event);

  void onMemberVideoMuteChanged(MemberToggleEvent event);

  void onMemberScreenShareStateChanged(MemberToggleEvent event);

  void onReceiveChatroomMessages(RoomMessageEvent messages);

  void onMemberWhiteboardShareStateChanged(MemberToggleEvent event);

  void onWhiteboardError(CommonEvent event);

  void onRoomLockStateChanged(RoomToggleEvent event);

  void onRoomLiveStateChanged(CommonEvent event);

  void onRtcAudioVolumeIndication(RtcVolumeEvent event);

  void onRtcAudioOutputDeviceChanged(CommonEvent event);
}
@HostApi(dartHostTestHandler: "RoomChatControllerApi")
abstract class RoomChatControllerApi{
  @async
  SimpleResponse joinChatroom(String roomUuid);

  @async
  SimpleResponse leaveChatroom(String roomUuid);

  @async
  SimpleResponse sendBroadcastTextMessage(String roomUuid,String message);

  @async
  SimpleResponse sendDirectTextMessage(String roomUuid,String userUuid,String message);

  @async
  SimpleResponse sendGroupTextMessage(String roomUuid,List<String> userUuids,String message);

  @async
  ChatRoomMembersResponse fetchChatRoomMembers(String roomUuid,int queryType,int limit);
}

class RoomLiveStreamTaskInfo {
  final String taskID;
  final String streamURL;
  final bool serverRecordEnabled;
  final String? extraInfo;
  final RoomLiveStreamLayout? layout;
  final RoomLiveConfig? config;
  final int lsMode;

  RoomLiveStreamTaskInfo({
    required this.taskID,
    required this.streamURL,
    required this.serverRecordEnabled,
    required this.layout,
    this.extraInfo,
    required this.config,
    required this.lsMode,
  });
}

class RoomLiveStreamLayout {
  final int width;
  final int height;
  final int backgroundColor;
  final List<RoomLiveStreamUserTranscoding?>? users;
  final RoomLiveStreamImageInfo? bgImage;

  RoomLiveStreamLayout({
    required this.width,
    required this.height,
    required this.backgroundColor,
    required this.users,
    required this.bgImage,
  });
}

class RoomLiveStreamUserTranscoding {
  final String userUuid;
  final bool videoPush;
  final int x;
  final int y;
  final int width;
  final int height;
  final bool audioPush;
  final int zOrder;
  final int adaption;

  RoomLiveStreamUserTranscoding({
    required this.userUuid,
    required this.x,
    required this.y,
    required this.width,
    required this.height,
    required this.videoPush,
    required this.audioPush,
    required this.zOrder,
    required this.adaption,
  });
}

class RoomLiveStreamImageInfo {
  final String? url;
  final int x;
  final int y;
  final int width;
  final int height;

  RoomLiveStreamImageInfo({
    this.url,
    required this.x,
    required this.y,
    required this.width,
    required this.height,
  });
}

class RoomLiveConfig {
  final bool singleVideoPassthrough;
  final int audioBitrate;
  final int sampleRate;
  final int channels;
  final int audioCodecProfile;

  RoomLiveConfig({
    required this.singleVideoPassthrough,
    required this.audioBitrate,
    required this.sampleRate,
    required this.channels,
    required this.audioCodecProfile,
  });
}

@HostApi(dartHostTestHandler: "RoomRtcControllerApi")
abstract class RoomRtcControllerApi{

  @async
  SimpleResponse joinRtcChannel(String roomUuid);

  @async
  SimpleResponse leaveRtcChannel(String roomUuid);

  @async
  SimpleResponse muteMyAudio(String roomUuid);

  @async
  SimpleResponse unmuteMyAudio(String roomUuid);

  @async
  SimpleResponse muteMemberAudio(String roomUuid,String uuid);

  @async
  SimpleResponse unmuteMemberAudio(String roomUuid,String uuid);

  @async
  SimpleResponse muteMemberVideo(String roomUuid,String uuid);

  @async
  SimpleResponse unmuteMemberVideo(String roomUuid,String uuid);

  @async
  SimpleResponse unmuteMyVideo(String roomUuid);

  @async
  SimpleResponse muteMyVideo(String roomUuid);

  @async
  SimpleResponse stopScreenShare(String roomUuid);

  @async
  SimpleResponse startScreenShare(StartScreenShareRequest request);

  @async
  SimpleResponse stopMemberScreenShare(String roomUuid,String uuid);

  @async
  SimpleResponse subscribeRemoteVideoStream(String roomUuid,String userUuid,bool subscribe);

  @async
  SimpleResponse subscribeRemoteSubStreamVideo(String roomUuid,String userUuid,bool subscribe);

  @async
  SimpleResponse setSpeakerphoneOn(String roomUuid, bool on);

  @async
  SimpleResponse isSpeakerphoneOn(String roomUuid,);

  @async
  SimpleResponse stopAudioDump(String roomUuid,);

  @async
  SimpleResponse startAudioDump(String roomUuid, int type);

  @async
  SimpleResponse enableAudioVolumeIndication(String roomUuid, bool enable, int interval);

  @async
  SimpleResponse startRtcChannelMediaRelay(String roomUuid);

  @async
  SimpleResponse stopRtcChannelMediaRelay(String roomUuid);

  @async
  SimpleResponse enableEarBack(String roomUuid, bool enable, int volume);

  @async
  SimpleResponse startAudioMixing(String roomUuid, CreateAudioMixingOption option);

  @async
  SimpleResponse playEffect(String roomUuid, int id, CreateAudioEffectOption option);

  @async
  SimpleResponse stopAudioMixing(String roomUuid);

  @async
  SimpleResponse stopEffect(String roomUuid, int id);

  @async
  SimpleResponse setAudioMixingSendVolume(String roomUuid, int volume);

  @async
  SimpleResponse setAudioMixingPlaybackVolume(String roomUuid, int volume);

  @async
  SimpleResponse setEffectSendVolume(String roomUuid, int id, int volume);

  @async
  SimpleResponse setEffectPlaybackVolume(String roomUuid, int id, int volume);

  @async
  SimpleResponse stopAllEffects(String roomUuid);

  @async
  SimpleResponse adjustUserPlaybackSignalVolume(String roomUuid, String userUuid, int volume);

  @async
  SimpleResponse switchCamera();

  @async
  SimpleResponse addBeautyFilter(String path);

  @async
  SimpleResponse addBeautySticker(String path);

  @async
  SimpleResponse enableBeauty(bool isOpenBeauty);

  @async
  SimpleResponse removeBeautyFilter();

  @async
  SimpleResponse removeBeautySticker();

  @async
  SimpleResponse startBeauty();

  @async
  SimpleResponse stopBeauty();

  @async
  SimpleResponse setBeautyFilterLevel(double level);

  @async
  SimpleResponse setBeautyEffect(int beautyType, double level);
}

@HostApi(dartHostTestHandler: "PreviewRoomRtcControllerApi")
abstract class PreviewRoomRtcControllerApi {
  @async
  SimpleResponse startPreview();

  @async
  SimpleResponse stopPreview();

  @async
  SimpleResponse switchCamera();

  @async
  SimpleResponse addBeautyFilter(String path);

  @async
  SimpleResponse addBeautySticker(String path);

  @async
  SimpleResponse enableBeauty(bool isOpenBeauty);

  @async
  SimpleResponse removeBeautyFilter();

  @async
  SimpleResponse removeBeautySticker();

  @async
  SimpleResponse startBeauty();

  @async
  SimpleResponse stopBeauty();

  @async
  SimpleResponse setBeautyFilterLevel(double level);

  @async
  SimpleResponse setBeautyEffect(int beautyType, double level);
}

class StartScreenShareRequest {
  final String roomUuid;
  final int? videoProfile;
  final int? frameRate;
  final String? iosAppGroup;

  StartScreenShareRequest({required this.roomUuid, this.videoProfile, this.frameRate, this.iosAppGroup});
}


class VideoRendererMirrorRequest {
  final int textureId;
  final bool mirror;
  final String roomUuid;

  VideoRendererMirrorRequest(this.textureId, this.mirror, this.roomUuid);
}

class TextureIdRendererRequest {
  final String roomUuid;
  final String? userUuid;
  final int textureId;

  TextureIdRendererRequest(this.roomUuid, this.userUuid, this.textureId);
}

@HostApi(dartHostTestHandler: "RoomRtcRenderApi")
abstract class RoomRtcVideoRender {
  @async
  SimpleResponse createVideoRenderer(String roomUuid);

  @async
  SimpleResponse setupLocalVideoRenderer(int textureId);

  // Future<VoidResult> setupLocalSubStreamVideoRenderer(
  //     TextureIdRendererRequest textureIdRendererRequest);

  @async
  SimpleResponse setMirror(int textureId, bool mirror);

  @async
  SimpleResponse setupRemoteSubStreamVideoRenderer(String userUuid, int textureId);

  @async
  SimpleResponse setupRemoteVideoRenderer(String userUuid, int textureId);

  @async
  SimpleResponse disposeVideoRenderer(int textureId);
}

@HostApi(dartHostTestHandler: "RoomWhiteboardControllerApi")
abstract class RoomWhiteboardControllerApi{

  @async
  SimpleResponse startWhiteboardShare(String roomUuid);

  @async
  SimpleResponse stopWhiteboardShare(String roomUuid);

  @async
  SimpleResponse setEnableDraw(String roomUuid,bool enable);

  @async
  SimpleResponse stopMemberWhiteboardShare(String roomUuid,String userUuid);

}

@FlutterApi()
abstract class MessageEventSink{
  void onReceivePassThroughMessage(PassThroughMessageEvent messageEvent);
}

class PassThroughMessageEvent{
  final String? roomUuid;
  final String? senderUuid;
  final String data;
  final int subCmd;
  PassThroughMessageEvent(this.roomUuid, this.senderUuid, this.data,this.subCmd);
}

@HostApi()
abstract class MessageServiceApi {
  @async
  SimpleResponse sendPassThroughMessage(String roomUuid, String userUuid, String data);
}

class RoomLiveRequest {
  final String title;
  final int liveLayout;
  final String?  password;
  final List<String?>? liveUuids;
  String? extras;


  RoomLiveRequest(
      this.title,
      this.liveLayout,
      this.password,
      this.liveUuids,
      this.extras);
}

class RoomLiveInfo{
  final String? taskId;
  final String? pushUrl;
  final String? chatRoomId;
  final String? httpPullUrl;
  final String? rtmpPullUrl;
  final String? hlsPullUrl;
  final String? title;
  final int?  liveLayout;
  final String? password;
  final List<String?>? liveUuids;
  int? state;
  String? extras;


  RoomLiveInfo(
      this.title,
      this.liveUuids,
      this.password,
      this.liveLayout,
      this.chatRoomId,
      this.hlsPullUrl,
      this.httpPullUrl,
      this.pushUrl,
      this.rtmpPullUrl,
      this.taskId,
      this.state,
      this.extras,
      );
}

class RoomLiveResponse {
  final int code;

  final String? msg;

  final RoomLiveInfo? liveInfo;

  RoomLiveResponse(this.code, this.msg, this.liveInfo);
}

@HostApi(dartHostTestHandler: "RoomLiveControllerApi")
abstract class RoomLiveControllerApi{

  @async
  RoomLiveResponse startLive(String roomUuid, RoomLiveRequest request);

  @async
  SimpleResponse stopLive(String roomUuid);

  @async
  RoomLiveResponse updateLive(String roomUuid,RoomLiveRequest request);

  @async
  RoomLiveResponse getLiveInfo(String roomUuid);

  @async
  SimpleResponse addLiveStreamTask(String roomUuid, RoomLiveStreamTaskInfo taskInfo);

  @async
  SimpleResponse updateLiveStreamTask(String roomUuid, RoomLiveStreamTaskInfo taskInfo);

  @async
  SimpleResponse removeLiveStreamTask(String roomUuid, String taskID);

}

class ChatRoomMember{
  final String? nick;
  final String? avatar;


  ChatRoomMember(
      this.nick,
      this.avatar,
      );
}

class ChatRoomMembersResponse {
  final int code;

  final String? msg;

  final List<ChatRoomMember?> chatRoomMembers;

  ChatRoomMembersResponse(this.code, this.msg, this.chatRoomMembers);
}

